/* global MathJax */
import React, { Component } from 'react'
import marked from 'marked'
import PropTypes from 'prop-types'

const DEFAULT_MATHJAX_CONF = {
  tex2jax: { inlineMath: [['$', '$']] },
}

const DEFAULT_MARKED_CONF = {
  gfm: true,
  tables: true,
  breaks: false,
  smartLists: true,
  smartypants: false
}

function merge(def, custom) {
  for (const key of Object.keys(custom)) {
    if (typeof def[key] === 'object') {
      def[key] = merge(def[key], custom[key])
    } else {
      def[key] = custom[key]
    }
  }
  return def
}

export default class MathDown extends Component {
  static propTypes = {
    jaxConfig: PropTypes.object,
    markedConfig: PropTypes.object,
    value: PropTypes.string,
  }

  constructor(props) {
    super(props)

    let markedConfig = DEFAULT_MARKED_CONF
    let jaxConfig = DEFAULT_MATHJAX_CONF
    if (props.markedConfig) {
      markedConfig = merge(markedConfig, props.markedConfig)
    }
    if (props.jaxConfig) {
      jaxConfig = merge(jaxConfig, props.jaxConfig)
    }
    // IMPORTANT, because we do MathJax before markdown
    markedConfig.sanitize = false
    marked.setOptions(markedConfig)
    MathJax.Hub.Config(jaxConfig)
  }

  componentDidMount() {
    return this.queue()
  }

  componentDidUpdate() {
    return this.queue()
  }

  queue() {
    MathJax.Hub.Queue(
      ['Typeset', MathJax.Hub, this.buffer],
      this.renderMarkdown
    )
  }

  setNode = (key, node) => {
    this[key] = node
  }

  renderMarkdown = () => {
    // textify
    const text = Array.from(this.buffer.childNodes).map(child => {
      if (child.nodeType === 3) {
        // TextNode may contain characters like (&), (<), or (>)
        // innerHTML returns these characters as &amp, &lt and &gt respectively
        return child.nodeValue
      } else {
        return child.outerHTML
      }
    }).join('')
    const result = marked(text)
    // no need to trigger render() again
    this.node.innerHTML = result
    // FIXME: How can I get rid of the redundant data?
    //this.buffer.innerHTML = ''
  }

  render() {
    const { value, jaxConfig, markedConfig, ...others } = this.props
    return (
      <div {...others}>
      <div
        style={{ display: 'none' }}
        ref={this.setNode.bind(this, 'buffer')}
        dangerouslySetInnerHTML={{__html: value }}
      />
      <div ref={this.setNode.bind(this, 'node')} />
      </div>
    )
  }
}
